# Support the following interface to the outside world.
#  Constructor can accept three different sets of arguments:
# \begin{enumerate}
#  \item  #(a, b, c)#
#  \item #(m, b)#: slope and intercept
# \item #(p1, p2)#: two points
# \end{enumerate}
# In any case, it should arrive at an internal representation of #(a,b,c)#
#\begin{tabbing}
# #self.distToPoint()#xx\=  Returns the point added most recently\kill
# #self.distToPoint(p)# \>  Returns the perpendicular distance from point #p# to the line\\
#\end{tabbing}


import math

class Line:
    def __init__(self, constructorType, args):
        if constructorType == "abc":
            self.initializeABC(args)
        elif constructorType == "mb":
            self.initializeMB(args)
        elif constructorType == "points":
            self.initializePoints(args)
        else:
            raise("Unknown constructor type")

    def initializeABC(self, (a,b,c)):
        self.a = a
        self.b = b
        self.c = c

    def initializeMB(self, (m, b)):
        self.a = m
        self.b = -1
        self.c = b

    def initializePoints(self, (p1, p2)):
        self.a = p2.y - p1.y
        self.b = p1.x - p2.x
        self.c = p2.x*p1.y - p1.x*p2.y
        
    def distToPoint(self, p):
        return abs(self.a * p.x + self.b * p.y + self.c)/math.sqrt(self.a**2 + self.b**2)

    # The line drawer doesn't know about this class, so sometimes we
    # have to make a tuple
    def abcTuple(self):
        return (self.a, self.b, self.c)

