# Support the following interface to the outside world.
#\begin{tabbing}
# #self.mostRecentPoint()#xx\=  Returns the point added most recently\kill
# #LineFitter()#      \>Makes a new line fitter with no points\\
# #self.clear()#      \>Empties out the set of points to be fit\\
# #self.mostRecentPoint()#  \>Returns the point added most recently\\
# #self.push(p)#       \>Adds a point p to the list\\
# #self.pop()#         \>Removes the oldest point from the list\\
# #self.line#          \>The best-fit line to the current set of points\\
# #self.maxError()#    \>The maximum perpendicular distance of one of the\\
#                     \>set of points to the best fit line.\\
#\end{tabbing}

import math
import point
reload(point)
from point import *
import line
reload(line)
from line import *

class LineFitter:

    def __init__(self):
        self.clear()

    def clear(self):
        self.points = []
        self.n = 0
        self.line = None

    def mostRecentPoint(self):
        return self.points[-1]

    # Keep the best fine line updated, whenever the list of points changes.
    def push(self, p):
        self.points.append(p)
        self.n += 1
        self.line = self.bestPerpFit()

    def pop(self):
        self.points.pop(0)
        self.n -= 1
        self.line = self.bestPerpFit()

    def maxError(self):
        # Your code here

    # Sum squared distance from list of points to line
    def sse(self, line):
        return sum([line.distToPoint(p)**2 for p in self.points])

    def bestPerpFit(self):
        sum_x = 0.0
        sum_y = 0.0
        sum_xx = 0.0
        sum_yy = 0.0
        sum_xy = 0.0
        for p in self.points:
            sum_x = sum_x + p.x
            sum_y = sum_y + p.y
            sum_xx = sum_xx + p.x * p.x
            sum_yy = sum_yy + p.y * p.y
            sum_xy = sum_xy + p.x * p.y
        if self.n <= 1:
            # not enough points
            return None
        xmean = sum_x / self.n
        ymean = sum_y / self.n
        # should be divided by n**2, but it cancels
        cov = sum_xy*self.n - sum_x*sum_y 
        xvar = self.n*sum_xx - sum_x**2   
        yvar = self.n*sum_yy - sum_y**2   
        numer = yvar - xvar
        if near(cov,0.0) and near(xvar,yvar):
            # points are in a 4-way symmetric blob
            print "Error: no line fit"
            return None
        elif near(cov,0.0) and yvar > xvar:
            # vertical line
            return Line("abc", (1, 0, -xmean))
        elif near(cov,0.0) and xvar > yvar:
            # horizontal line
            return Line("abc", (0, 1, -ymean))
        else:
            h = numer / cov
            a1 = (h + math.sqrt(h**2 + 4))/2
            a2 = (h - math.sqrt(h**2 + 4))/2
            c1 = -a1 * xmean + ymean
            c2 = -a2 * xmean + ymean
            l1 = Line("abc", (a1, -1, c1))
            l2 = Line("abc", (a2, -1, c2))
            # Pick the value of (a,c) that minimizes squared error
            if self.sse(l1) < self.sse(l2):
                return l1
            else:
                return l2

# Never test real values for equality!!
def near(v1,v2):
    return abs(v1-v2) < .0000001

