# Support the following interface to the outside world.
#\begin{tabbing}
# #self.mostRecentPoint()#xxxxxxxx\=  Returns the point added most recently\kill
# #LineSensor(sensorIndex, window)#\>Makes a new line sensor, taking readings from sonar number #sensorIndex#; \\
# \>using #window# for debugging output\\
# #self.update(distances, pose)#  \>Given a vector of sonar readings and the current robot pose, \\
# \>update the state of the line sensor\\
# #self.read(pose)#  \>Returns a virtual sonar reading, by "shooting" a ray out of the sensor\\
# \>and seeing how far it goes before intersecting the line\\
# #self.isValid()#       \>Returns #True# if the readings are likely to be valid\\
#\end{tabbing}

from soarmath import intersection
from simulator import SONAR_INFO

import line
reload(line)
from line import *
import poseClass
reload(poseClass)
from poseClass import *
import lineTracker
reload(lineTracker)
from lineTracker import LineTracker
import point
reload(point)
from point import *

maxReading = 10.0

sonarPoses = [Pose(p) for p in SONAR_INFO]

class lineSensor:

    def __init__(self, sensorIndex, window):
        self.tracker = LineTracker(sensorIndex)
        self.sensorIndex = sensorIndex
        self.w = window

    # Update our internal state.  Pass in all the sonarDistances
    # because if we are averaging over space we might need to look at
    # more than one value.
    def update(self, distances, pose):
        def getSonarHit(d, pose):
            return pose.transform(Point(d,0))
        hits = map(getSonarHit, distances, sonarPoses)
        self.tracker.update(pose, hits, self.w)

    # Read a value
    def read(self, robotPose):
        # This part is really gross.  Need to find two points on this
        # line for Mike's intersection routine.  Better to use
        # something that intersects a line and a ray
        if self.tracker.currentLine == None:
            return maxReading
        else:
            (a,b,c) = self.tracker.currentLine.abcTuple()
            if abs(b) > .00001:
                lineP1 = Point(100,-(a*100.0+c)/b)
                lineP2 = Point(-100,(a*100.0-c)/b)
            elif abs(a) > .00001:
                lineP1 = Point(-(b*100.0-c)/a ,100)
                lineP2 = Point((b*100.0-c)/a ,-100)
            else:
                return maxReading

            localSonarPose = sonarPoses[self.sensorIndex]
            globalSonarPoint = robotPose.transform(localSonarPose.getPoint())
            globalSonarRayEnd = robotPose.transform(\
                              localSonarPose.transform(Point(3.0,0.0)))
            sonarLine = Line("points", (globalSonarPoint,
                                        globalSonarRayEnd))
            crossing = intersection((globalSonarPoint.xytuple(),
                                     globalSonarRayEnd.xytuple()),
                                    (lineP1.xytuple(),lineP2.xytuple()))
            if crossing == False:
                return 10.0
            else:
                if self.w != None:
                    self.w.drawPoint(crossing[0],crossing[1],color = "orange")
                return globalSonarPoint.distToPoint(Point(crossing[0],crossing[1]))


    def isValid(self):
        return self.tracker.isLineValid
    
